/**
 *  Copyright 2011 Carsten Gräf
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 */

package de.twenty11.skysail.common.responses;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.Validate;
import org.codehaus.jackson.annotate.JsonIgnore;

import de.twenty11.skysail.common.SkysailData;
import de.twenty11.skysail.common.navigation.Navigation;
import de.twenty11.skysail.common.pagination.Pagination;

/**
 * The root class for restful responses served by skysail server.
 * 
 * A skysail response provides:
 * 
 * - a success flag: did the request run ok on the server side? - a debug flag:
 * expose more data of the response to the user than usual? - a message: a
 * string providing some information about the response (or an error message if
 * appropriate) - origRequest/request: // TODO - parent - data: the actual data
 * (of type T) - filter: To be done - page: pagination information - current
 * 'page' of the (partial) resource representation - pageSize: pagination
 * information - how many elements fit on one 'page' - totalResults: pagination
 * information - how many elements are there altogether - contextPath: // TODO -
 * sortingRepresentation: a string like "s=0|1|2|0|0&" indicating how to sort by
 * columns - paramsToStrip: // TODO
 * 
 * The order of the fields in response (seems to) depend(s) on the order of the
 * fields in this class.
 * 
 * This class is used in {@link SkysailServerResource} to define the common
 * structure how skysail REST representations of resources look like. The plain
 * 'data' of the resource itself is augmented with the additional information
 * shown above in order to support pagination, filtering, sorting and the like.
 * 
 * Remark: This class is not abstract as of jackson serialization issues
 * 
 * @param <T>
 *            a type extending SkysailData
 */
public class SkysailResponse<T extends SkysailData> {

    /**
     * a message to be passed to the client.
     */
    private String message;

    /**
	 * Indicates if the request was successful. If not, the "message" field
	 * should contain an error description for the caller.
	 */
	private Boolean success = true;

    private Navigation navigation = new Navigation();

	/** LDAP search filter expression, see http://www.ietf.org/rfc/rfc1960.txt. */
	private String filter;
	
    /**
     * the actual data, of a type T extending SkysailData.
     */
	private T data = null;

    private Pagination pagination = new Pagination(); 

    /**
     * string representation how to sort the response data.
     */
    private String sortingRepresentation;

   /**
     * Indicates if the response presentation should provide additional
     * information for debugging. This is meant to be used in the presentation
     * layer, e.g. to show more of the response data than usual.
     */
    private Boolean debug = false;

	private final Set<String> paramsToStrip = new HashSet<String>();

	/**
	 * Default Constructor.
	 * 
	 * A couple of parameter names which are used internally by skysail are set to the
	 * list of parameters to strip when creating response pages.
	 */
	public SkysailResponse() {
		paramsToStrip.add("page");
		paramsToStrip.add("pageSize");
		paramsToStrip.add("toggleSorting");
		paramsToStrip.add("s");
	}

	// --- the getters and setters ---
	/**
	 * getter.
	 * 
	 * @return success or not
	 */
	public final Boolean getSuccess() {
		return success;
	}

	/**
	 * setter.
	 * 
	 * @param successful
	 *            true or false
	 */
	public final void setSuccess(final Boolean successful) {
		this.success = successful;
	}

	/**
	 * 
	 * @param shouldDebug
	 *            true or false
	 */
	// TODO extend so that the debug flag cannot be set to true on production
	// (for example)
	public void setDebug(final Boolean shouldDebug) {
		this.debug = shouldDebug;
	}

	/**
	 * @return
	 */
	public Boolean getDebug() {
		return debug;
	}

	/**
	 * The original request is stripped from some parameters like page and
	 * toggleSorting to facilitate the processing on the presentation layer.
	 * 
	 * @param origRequest
	 *            the last request received.
	 */
	public void setRequest(final String origRequest) {
		String url = stripDoubleAndEmptyParameters(origRequest);
        if (url == null) {
            return;
        }
		url = checkLastCharacter(url);
		for (String paramName : paramsToStrip) {
			url = url.replaceAll(paramName + "=[^&]*&?", "");
		}
		if (url.contains("?") && !url.endsWith("?")) {
			url = url + "&";
		} else if (!url.contains("?")) {
			url = url + "?";
		}
		this.navigation.setRequest(url);

	}

	private String checkLastCharacter(final String url) {
		
		Validate.notNull(url);
		
		String result = url;
		if (url.endsWith("?")) {
			result = url.substring(0, url.length() - 1);
		}
		if (url.endsWith("&")) {
			result = url.substring(0, url.length() - 1);
		}
		return result;
	}

	private String stripDoubleAndEmptyParameters(String req) {
        if (req == null) {
            return null;
        }

		String[] reqSplit = req.split("\\?");
		if (reqSplit.length <= 1) {
			return req;
		}
		String query = reqSplit[1];
		String[] parameterPairs = query.split("&");
		Map<String, String> parameterMap = new HashMap<String, String>();
		for (String parameterPair : parameterPairs) {
			String[] split = parameterPair.split("=");
			parameterMap.put(split[0], split.length > 1 ? split[1] : "");
		}
		StringBuffer sb = new StringBuffer(reqSplit[0] + "?");
		for (Map.Entry<String, String> entry : parameterMap.entrySet()) {
			if (entry.getValue() != null && !entry.getValue().trim().equals("")) {
				sb.append(entry.getKey()).append("=").append(entry.getValue())
						.append("&");
			}
		}
		return sb.toString();
	}

	/**
	 * getter.
	 * 
	 * @return the message
	 */
	public final String getMessage() {
		return message;
	}

	/**
	 * setter.
	 * 
	 * @param msg
	 *            info for the client
	 */
	public final void setMessage(final String msg) {
		this.message = msg;
	}

	/**
	 * getter.
	 * 
	 * @return the data
	 */
	public T getData() {
		return data;
	}

	/**
	 * setter.
	 * 
	 * @param mydata
	 *            set provided data
	 */
	public final void setData(final T mydata) {
		this.data = mydata;
	}

	public void setParent(final String url) {
		this.navigation.setParent(url);
	}

	public void setFilter(final String filter) {
		this.filter = filter;
	}

	@JsonIgnore
	public String getFilter() {
		return filter;
	}

	public void setContextPath(final String path) {
		this.navigation.setContextPath(path);
	}

	public String getContextPath() {
		return navigation.getContextPath();
	}
	
	public Pagination getPagination() {
        return pagination;
    }
	
	public Navigation getNavigation() {
        return navigation;
    }

	public void setSortingRepresentation(final String sorting) {
		this.sortingRepresentation = sorting;
	}

	public String getSortingRepresentation() {
		return sortingRepresentation;
	}

    public void setPageSize(int i) {
        this.pagination.setPageSize(i);
    }

    public void setPage(int i) {
        pagination.setPage(i);
    }

    public void setTotalResults(int i) {
        pagination.setTotalResults(i);
    }

}